#include "Graphics\Graphics.h"

// Color
WORD_VAL    _color;
// Clipping region control
SHORT _clipRgn;
// Clipping region borders
SHORT _clipLeft;
SHORT _clipTop;
SHORT _clipRight;
SHORT _clipBottom;


#define lo(w) ((unsigned char)(w))
#define hi(w) ((unsigned char)(((unsigned int)(w) >> 8)))

#define TFTDATA 1
#define TFTCMD  0


#define  RST   	LATDbits.LATD9       // LCD reset signal 
#define  RS		LATBbits.LATB15
#define  CS		LATDbits.LATD10
#define  WR		LATDbits.LATD4
#define  RD		LATDbits.LATD5

#define PMPWaitBusy()   while(PMMODEbits.BUSY); 
inline void DeviceSetCommand(){	RS = 0;}
inline void DeviceSetData(){RS = 1;}
inline void DeviceSelect(){PMADDR |=0x8000;}
inline void DeviceDeselect(){PMADDR &=0x7FFF;}
/////////////////////// LOCAL FUNCTIONS PROTOTYPES ////////////////////////////
void WriteData(BYTE byte1, BYTE byte0);
void SetAddress(SHORT x, SHORT y);

#define DELAY_1MS 16000/5  // for 16MIPS
void  DelayMs(WORD time){
unsigned delay;
	while(time--)
		for(delay=0; delay<DELAY_1MS; delay++);	
}
/*********************************************************************
* Macros:  WritePixel(data)
* Overview: Writes data
* PreCondition: none
* Input: data
* Output: none
* Side Effects: none
********************************************************************/
#define WritePixel(data) \
    DeviceWrite( ((WORD_VAL) data).v[1] & 0xF8 );        \
    DeviceWrite( ((data >> 3) & 0xFC) );                 \
    DeviceWrite( (((WORD_VAL) data).v[0] << 3) & 0xF8 );
/*********************************************************************
* Function:  void SetAddress(SHORT x, SHORT y)
*
* Overview: Writes address pointer.
*
* PreCondition: none
*
* Input: x - horizontal position
*        y - vertical position
*
* Output: none
*
* Side Effects: none
*
********************************************************************/
void SetAddress(SHORT x, SHORT y)
{
	SetReg(0x42,x);
	SetReg(0x43,hi(y));
	SetReg(0x44,lo(y));
}
/*********************************************************************
* Function:  void  SetReg(BYTE index, BYTE value)
* PreCondition: none
* Input: index - register number
*        value - value to be set
* Output: none
* Side Effects: none
* Overview: sets graphics controller register
* Note: none
********************************************************************/
void SetReg(BYTE index, BYTE value)
{
//	CS = 0;
	RS = 0;	
	DeviceWrite(index);
//	Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();
//	RS = 1;	
	DeviceWrite(value);
//	CS = 1;
}

void ReadReg(BYTE index)
{
BYTE value2;
	
//	CS = 0;
	RS = 0;

	DeviceWrite(index);
	Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();
	value2 = DeviceRead();
//	CS = 1;
	return value2;

}


inline void DeviceWrite(BYTE data)
{

	PMDIN1 = data;
	PMPWaitBusy();
//DelayMs (20);
}

/////////////////////////////
BYTE DeviceRead(){
BYTE value;
	value = PMDIN1;
	PMPWaitBusy();
//	PMCONbits.PMPEN = 0; // disable PMP
	value = PMDIN1;
//	PMCONbits.PMPEN = 1; // enable  PMP
	return value;
}



void WriteData(BYTE byte1, BYTE byte0)
{
//	CS = 0;
	RS = 1;
	PMDIN1 = byte1;
	PMPWaitBusy();

	PMDIN1 = byte0;
	PMPWaitBusy();

//	CS = 1;
	
}

/*********************************************************************
* Macros:  WritePixel(data)
* Overview: Writes data
* PreCondition: none
* Input: data
* Output: none
* Side Effects: none
********************************************************************/
#ifdef USE_16BIT_PMP
#define WritePixel(data)  DeviceWrite(data)
#else
#define WritePixel(data) \
    DeviceWrite( ((WORD_VAL) data).v[1] & 0xF8 );        \
    DeviceWrite( ((data >> 3) & 0xFC) );                 \
    DeviceWrite( (((WORD_VAL) data).v[0] << 3) & 0xF8 );
#endif

/*********************************************************************
* Function: void PutPixel(SHORT x, SHORT y)
*
* PreCondition: none
*
* Input: x,y - pixel coordinates
*
* Output: none
*
* Side Effects: none
*
* Overview: puts pixel
*
* Note: none
*
********************************************************************/
void PutPixel(SHORT x, SHORT y){
DWORD_VAL address;
    if(_clipRgn){
        if(x<_clipLeft)
            return;
        if(x>_clipRight)
            return;
        if(y<_clipTop)
            return;
        if(y>_clipBottom)
            return;
    }
    SetAddress(x, y);
    WriteData(_color.v[1],_color.v[0]);   
}




void TFTPowerOn (void)
{
	SetReg(REG_CONTROL_2, 0x10);
	SetReg(REG_CONTROL_1, 0xA0);
	SetReg(REG_RESET, 0x01);
	DelayMs(10);
	SetReg(REG_RESET, 0x00);
	SetReg(REG_OSC_RESIST_CON, 0x04);

  	SetReg(0x59, 0x01); //Test register setting enable 
  	SetReg(0x60, 0x22); //Test register setting 
  	SetReg(0x59, 0x00); //Test register setting disable 

	SetReg(REG_POWER_SUPPLY_CON14, 0x18);
	SetReg(REG_POWER_SUPPLY_CON3, 0x05);
	SetReg(REG_POWER_SUPPLY_CON12, 0x05);
	SetReg(REG_POWER_SUPPLY_CON2, 0x90);

	DelayMs(10);

	SetReg(REG_POWER_SUPPLY_CON5, 0x73);
	SetReg(0x24, 0x74); 
	SetReg(REG_POWER_SUPPLY_CON8, 0x01);
	SetReg(REG_POWER_SUPPLY_CON1, 0xC1);

	DelayMs(10);

	SetReg(REG_POWER_SUPPLY_CON1, 0xE1);
	SetReg(REG_POWER_SUPPLY_CON1, 0xF1);

// wait for >60ms
	DelayMs(60);		

	SetReg(REG_POWER_SUPPLY_CON1, 0xF5);	

// wait for >60ms
	DelayMs(60);

	SetReg(REG_POWER_SUPPLY_CON4, 0x0B);	

// wait for >10ms
	DelayMs(10);

	SetReg(REG_POWER_SUPPLY_CON4, 0x0B);	
	SetReg(REG_POWER_SUPPLY_CON9, 0x11);
	SetReg(REG_POWER_SUPPLY_CON10, 0x10);		
	SetReg(REG_POWER_SUPPLY_CON8, 0x81);	

// wait for >10ms
	DelayMs(40);
}


void TFTDisplayOn (void)
{
	SetReg(REG_CONTROL_1, 0x20);	
	SetReg(0x01, 0xD0);
	SetReg(0x02, 0x00);
	SetReg(0x01, 0x10); 
	SetReg(0x02, 0x00);
//	SetReg(REG_GATE_OUT_CON, 0x01);
	DelayMs(60);
//	SetReg(REG_CONTROL_1, 0x20);		
}

void ResetDevice( void)
{
	BYTE value1;
	int i,j;
// PMP intialization 		
	RST = 0;			// Put TFT to reset	
	TRISDbits.TRISD9 = 0; 
	TRISBbits.TRISB15 = 0;
//	CS = 0;
	TRISDbits.TRISD10 = 0;	
	LATAbits.LATA14=0;		/* PSX = 0 */
	LATAbits.LATA15=0;		/* RGB/CPU=0 */

	PMMODE = 0;
//    PMPEN = 0;
    PMCON = 0;
    PMMODEbits.MODE = 2	;
 	PMMODEbits.WAITB = 1; 
    PMMODEbits.WAITM = 3; 
    PMMODEbits.WAITE = 0; 
    PMMODEbits.MODE16 = 0;                  // 8 bit mode
    PMCONbits.PTRDEN = 1;                   // enable RD line
    PMCONbits.PTWREN = 1;                   // enable WR line
    PMCONbits.CSF = 1;  
    PMCONbits.PMPEN = 1;                    // enable PMP
	PMPEN = 0x8000;
	PMADDR = 0x8000;

    DelayMs(40);
	RST = 1;			/* LSI starts       */
 	DelayMs(400);

	TFTPowerOn ();

/***********Void Chip_Set************/   
  SetReg(0x9D, 0x04); 
  SetReg(0xC0, 0x00); 
  SetReg(0xC1, 0x00); //R193:BGR bit='0' 
  SetReg(0x0E, 0x00); 
  SetReg(0x0F, 0x00); 
  SetReg(0x10, 0x00); 
  SetReg(0x11, 0x00); 
  SetReg(0x12, 0x00); 
  SetReg(0x13, 0x00);
  SetReg(0x14, 0x00); 
  SetReg(0x15, 0x00); 
  SetReg(0x16, 0x00); 
  SetReg(0x17, 0x00); 
  SetReg(0x34, 0x01); 
  SetReg(0x35, 0x00); 
  SetReg(0x4B, 0x00); 
  SetReg(0x4C, 0x00); 
  SetReg(0x4E, 0x00); 
  SetReg(0x4F, 0x00); 
  SetReg(0x50, 0x00); 

  SetReg(0x3C, 0x00); 
  SetReg(0x3D, 0x00); 
  SetReg(0x3E, 0x01); 
  SetReg(0x3F, 0x3F); 
  SetReg(0x40, 0x01); 
  SetReg(0x41, 0x09); 

  SetReg(0x42, 0x00); 
  SetReg(0x43, 0x00); 
  SetReg(0x44, 0x00); 
  SetReg(0x45, 0x00); 
  SetReg(0x46, 0xEF); 
  SetReg(0x47, 0x00); 
  SetReg(0x48, 0x00); 
  SetReg(0x49, 0x01); 
  SetReg(0x4A, 0x3F); 

  SetReg(0x1D, 0x08); //R29:Gate scan direction setting 
  SetReg(0x86, 0x00);   
  SetReg(0x87, 0x30); 
  SetReg(0x88, 0x02); 
  SetReg(0x89, 0x05); 
  SetReg(0x8D, 0x01); //R141:Register set-up mode for one line clock number 
  SetReg(0x8B, 0x30); //R139:One line SYSCLK number in one-line scan 
  SetReg(0x33, 0x01); //R51:N line inversion setting 
  SetReg(0x37, 0x01); //R55:Scanning method setting 
  SetReg(0x76, 0x00); 

/**********Void Gamma_Set************/ 
  SetReg(0x8F, 0x00); 
  SetReg(0x90, 0x77); 
  SetReg(0x91, 0x07); 
  SetReg(0x92, 0x54); 
  SetReg(0x93, 0x07); 
  SetReg(0x94, 0x00); 
  SetReg(0x95, 0x77); 
  SetReg(0x96, 0x45); 
  SetReg(0x97, 0x00); 
  SetReg(0x98, 0x06); 
  SetReg(0x99, 0x03); 
  SetReg(0x9A, 0x00); 	

  SetReg(0x3B, 0x01);
	DelayMs(40);

	TFTDisplayOn ();

//	SetAddress (0, 0);
//	DeviceRead ();
	ReadReg(0x01);	
	SetColor(BLACK);
	ClearDevice();
//	ClearDevice();
//	PutPixel(100, 100);

} // LCDinit
/*********************************************************************
* Function: void ClearDevice(void)
*
* PreCondition: none
*
* Input: none
*
* Output: none
*
* Side Effects: none
*
* Overview: clears screen with current color 
*
* Note: none
*
********************************************************************/
#ifdef USE_DRV_CLEARDEVICE
void ClearDevice(void){
DWORD_VAL     address;

	SetAddress(0,0);
    for(address.Val=0; address.Val<(DWORD)(GetMaxX()+1)*(GetMaxY()+1); address.Val++)
	{
        WriteData(_color.v[1],_color.v[0]);
    }
}
#endif

/*********************************************************************
* Function: WORD Bar(SHORT left, SHORT top, SHORT right, SHORT bottom)
*
* PreCondition: none
*
* Input: left,top - top left corner coordinates,
*        right,bottom - bottom right corner coordinates
*
* Output: none
*
* Side Effects: none
*
* Overview: draws rectangle filled with current color
*
* Note: none
*
********************************************************************/
#ifdef USE_DRV_BAR
WORD Bar(SHORT left, SHORT top, SHORT right, SHORT bottom)
{
    register SHORT  x, y;

    #ifndef USE_NONBLOCKING_CONFIG
    while(IsDeviceBusy() != 0);

    /* Ready */
    #else
    if(IsDeviceBusy() != 0)
        return (0);
    #endif
    if(_clipRgn)
    {
        if(left < _clipLeft)
            left = _clipLeft;
        if(right > _clipRight)
            right = _clipRight;
        if(top < _clipTop)
            top = _clipTop;
        if(bottom > _clipBottom)
            bottom = _clipBottom;
    }

    DeviceSelect();
    for(y = top; y < bottom + 1; y++)
    {
        SetAddress(left, y);
        for(x = left; x < right + 1; x++)
        {
            WriteData(_color.v[1],_color.v[0]);
        }
    }

    DeviceDeselect();
    return (1);
}
#endif





