#ifndef _HX8312_H
    #define _HX8312_H

    #include <p24FJ128GA010.h>
	#include "Graphics\Graphics.h"
    #include "GenericTypeDefs.h"
    #include "GraphicsConfig.h"

/*********************************************************************
* Overview: Additional hardware-accelerated functions can be implemented
*           in the driver. These definitions exclude the PutPixel()-based
*           functions in the primitives layer (Primitive.c file) from compilation.
*********************************************************************/

// Define this to implement Font related functions in the driver.
	//#define USE_DRV_FONT
// Define this to implement Line function in the driver.
	//#define USE_DRV_LINE
// Define this to implement Circle function in the driver.
	//#define USE_DRV_CIRCLE
// Define this to implement FillCircle function in the driver.
	//#define USE_DRV_FILLCIRCLE
// Define this to implement Bar function in the driver.
    #define USE_DRV_BAR

// Define this to implement ClearDevice function in the driver.
    #define USE_DRV_CLEARDEVICE

// Define this to implement PutImage function in the driver.
    #define USE_DRV_PUTIMAGE

    #ifndef DISP_HOR_RESOLUTION
        #error DISP_HOR_RESOLUTION must be defined in HardwareProfile.h
    #endif
    #ifndef DISP_VER_RESOLUTION
        #error DISP_VER_RESOLUTION must be defined in HardwareProfile.h
    #endif
    #ifndef COLOR_DEPTH
        #error COLOR_DEPTH must be defined in HardwareProfile.h
    #endif
    #ifndef DISP_ORIENTATION
        #error DISP_ORIENTATION must be defined in HardwareProfile.h
    #endif

/*********************************************************************
* Overview: Horizontal and vertical screen size.
*********************************************************************/
    #if (DISP_HOR_RESOLUTION != 240)
        #error This driver does not support this resolution. Horisontal resolution must be 240 pixels.
    #endif
    #if (DISP_VER_RESOLUTION != 320)
        #error This driver does not support this resolution. Vertical resolution must be 320 pixels.
    #endif

/*********************************************************************
* Overview: Display orientation.
*********************************************************************/
    #if (DISP_ORIENTATION != 0) && (DISP_ORIENTATION != 90)
        #error This driver does not support this orientation.
    #endif

/*********************************************************************
* Overview: Color depth.
*********************************************************************/
    #if (COLOR_DEPTH != 16)
        #error This driver does not support this color depth. It should be 16.
    #endif

/*********************************************************************
* Overview: Clipping region control codes to be used with SetClip(...)
*           function. 
*********************************************************************/
    #define CLIP_DISABLE    0   // Disables clipping.
    #define CLIP_ENABLE     1   // Enables clipping.

/*********************************************************************
* Overview: Some basic colors definitions.
*********************************************************************/
    #define BLACK           RGB565CONVERT(0, 0, 0)
    #define BRIGHTBLUE      RGB565CONVERT(0, 0, 255)
    #define BRIGHTGREEN     RGB565CONVERT(0, 255, 0)
    #define BRIGHTCYAN      RGB565CONVERT(0, 255, 255)
    #define BRIGHTRED       RGB565CONVERT(255, 0, 0)
    #define BRIGHTMAGENTA   RGB565CONVERT(255, 0, 255)
    #define BRIGHTYELLOW    RGB565CONVERT(255, 255, 0)
    #define BLUE            RGB565CONVERT(0, 0, 128)
    #define GREEN           RGB565CONVERT(0, 128, 0)
    #define CYAN            RGB565CONVERT(0, 128, 128)
    #define RED             RGB565CONVERT(128, 0, 0)
    #define MAGENTA         RGB565CONVERT(128, 0, 128)
    #define BROWN           RGB565CONVERT(255, 128, 0)
    #define LIGHTGRAY       RGB565CONVERT(128, 128, 128)
    #define DARKGRAY        RGB565CONVERT(64, 64, 64)
    #define LIGHTBLUE       RGB565CONVERT(128, 128, 255)
    #define LIGHTGREEN      RGB565CONVERT(128, 255, 128)
    #define LIGHTCYAN       RGB565CONVERT(128, 255, 255)
    #define LIGHTRED        RGB565CONVERT(255, 128, 128)
    #define LIGHTMAGENTA    RGB565CONVERT(255, 128, 255)
    #define YELLOW          RGB565CONVERT(255, 255, 128)
    #define WHITE           RGB565CONVERT(255, 255, 255)

    #define GRAY0           RGB565CONVERT(224, 224, 224)
    #define GRAY1           RGB565CONVERT(192, 192, 192)
    #define GRAY2           RGB565CONVERT(160, 160, 160)
    #define GRAY3           RGB565CONVERT(128, 128, 128)
    #define GRAY4           RGB565CONVERT(96, 96, 96)
    #define GRAY5           RGB565CONVERT(64, 64, 64)
    #define GRAY6           RGB565CONVERT(32, 32, 32)


// Color
extern WORD_VAL _color;
/*********************************************************************
* Overview: Clipping region control and border settings.
*********************************************************************/
extern SHORT    _clipRgn;					// Clipping region enable control
extern SHORT    _clipLeft;					// Left clipping region border
extern SHORT    _clipTop;					// Top clipping region border
extern SHORT    _clipRight;					// Right clipping region border
extern SHORT    _clipBottom;				// Bottom clipping region border
/*********************************************************************
* Function:  void ResetDevice()
* Overview: Initializes LCD module.
* PreCondition: none
* Input: none
* Output: none
* Side Effects: none
********************************************************************/
void    ResetDevice(void);
/*********************************************************************
* Macros:  GetMaxX()
* Overview: Returns maximum horizontal coordinate.
* PreCondition: none
* Input: none
* Output: Maximum horizontal coordinate.
* Side Effects: none
********************************************************************/
    #if (DISP_ORIENTATION == 90)
        #define GetMaxX()   (DISP_VER_RESOLUTION - 1)
    #elif (DISP_ORIENTATION == 0)
        #define GetMaxX()   (DISP_HOR_RESOLUTION - 1)
    #endif
/*********************************************************************
* Macros:  GetMaxY()
* Overview: Returns maximum vertical coordinate.
* PreCondition: none
* Input: none
* Output: Maximum vertical coordinate.
* Side Effects: none
********************************************************************/
    #if (DISP_ORIENTATION == 90)
        #define GetMaxY()   (DISP_HOR_RESOLUTION - 1)
    #elif (DISP_ORIENTATION == 0)
        #define GetMaxY()   (DISP_VER_RESOLUTION - 1)
    #endif

/*********************************************************************
* Macros:  SetColor(color)
* Overview: Sets current drawing color.
* PreCondition: none
* Input: color - Color coded in 5:6:5 RGB format.
* Output: none
* Side Effects: none
********************************************************************/
#define SetColor(color) _color.Val = color;
/*********************************************************************
* Macros:  GetColor()
* Overview: Returns current drawing color.
* PreCondition: none
* Input: none
* Output: Color coded in 5:6:5 RGB format.
* Side Effects: none
********************************************************************/
#define GetColor() _color.val
/*********************************************************************
* Function: void PutPixel(SHORT x, SHORT y)
* Overview: Puts pixel with the given x,y coordinate position.
* PreCondition: none
* Input: x - x position of the pixel.
*		 y - y position of the pixel.
* Output: none
* Side Effects: none
********************************************************************/
void    PutPixel(SHORT x, SHORT y);
/*********************************************************************
* Function: WORD GetPixel(SHORT x, SHORT y)
* Overview: Returns pixel color at the given x,y coordinate position.
* PreCondition: none
* Input: x - x position of the pixel.
*		 y - y position of the pixel.
* Output: pixel color
* Side Effects: none
********************************************************************/
WORD    GetPixel(SHORT x, SHORT y);
/*********************************************************************
* Macros: SetClipRgn(left, top, right, bottom)
*
* Overview: Sets clipping region.
*
* PreCondition: none
*
* Input: left - Defines the left clipping region border.
*		 top - Defines the top clipping region border.
*		 right - Defines the right clipping region border.
*	     bottom - Defines the bottom clipping region border.
*
* Output: none
*
* Side Effects: none
*
********************************************************************/
#define SetClipRgn(left, top, right, bottom) \
    _clipLeft = left;                            \
    _clipTop = top;                              \
    _clipRight = right;                          \
    _clipBottom = bottom;
/*********************************************************************
* Macros: GetClipLeft()
* Overview: Returns left clipping border.
* PreCondition: none
* Input: none
* Output: Left clipping border.
* Side Effects: none
********************************************************************/
#define GetClipLeft()   _clipLeft
/*********************************************************************
* Macros: GetClipRight()
* Overview: Returns right clipping border.
* PreCondition: none
* Input: none
* Output: Right clipping border.
* Side Effects: none
********************************************************************/
#define GetClipRight()  _clipRight
/*********************************************************************
* Macros: GetClipTop()
* Overview: Returns top clipping border.
* PreCondition: none
* Input: none
* Output: Top clipping border.
* Side Effects: none
********************************************************************/
#define GetClipTop()    _clipTop
/*********************************************************************
* Macros: GetClipBottom()
* Overview: Returns bottom clipping border.
* PreCondition: none
* Input: none
* Output: Bottom clipping border.
* Side Effects: none
********************************************************************/
#define GetClipBottom() _clipBottom
/*********************************************************************
* Macros: SetClip(control)
* Overview: Enables/disables clipping.
* PreCondition: none
* Input: control - Enables or disables the clipping.
*			- 0: Disable clipping
*			- 1: Enable clipping
* Output: none
* Side Effects: none
********************************************************************/
#define SetClip(control)    _clipRgn = control;
/*********************************************************************
* Macros: IsDeviceBusy()
* Overview: Returns non-zero if LCD controller is busy 
*           (previous drawing operation is not completed).
* PreCondition: none
* Input: none
* Output: Busy status.
* Side Effects: none
********************************************************************/
#define IsDeviceBusy()  0
/*********************************************************************
* Macros: SetPalette(colorNum, color)
* Overview:  Sets palette register.
* PreCondition: none
* Input: colorNum - Register number.
*        color - Color.
* Output: none
* Side Effects: none
********************************************************************/
#define SetPalette(colorNum, color)
/*********************************************************************
* Function:  void  DelayMs(WORD time)
* Overview: Delays execution on time specified in milliseconds.
*           The delay is correct only for 16MIPS.
* PreCondition: none
* Input: time - Delay in milliseconds.
* Output: none
* Side Effects: none
********************************************************************/
void DelayMs(WORD time);

// hx8312A registers names definitions
    #define REG_CONTROL_1		            0x00	/* 1 byte */	
    #define REG_CONTROL_2		            0x01
    #define REG_RGB				            0x02
    #define REG_RESET			            0x03
    #define REG_RAM_ACCESS	                0x05
    #define REG_DATA_REVERSE		        0x06
    #define REG_DISP_SIZE_CONT		        0x0d
    #define REG_DISP_PART_COLOR1	        0x0e
    #define REG_DISP_PART_COLOR2            0x0f
    #define REG_DISP_WIND_START1 	        0x10	/* 2 bytes */
    #define REG_DISP_WIND_START2            0x12
    #define REG_DISP_WIND_LINENUMB1         0x14
    #define REG_DISP_WIND_LINENUMB2		    0x16
    #define REG_POWER_SUPPLY_CON1		    0x18
    #define REG_POWER_SUPPLY_CON2           0x19
    #define REG_POWER_SUPPLY_CON3		    0x1A
    #define REG_POWER_SUPPLY_CON4		    0x1B
	#define REG_POWER_SUPPLY_CON5		    0x1C
	#define REG_POWER_SUPPLY_CON6		    0x1D
	#define REG_POWER_SUPPLY_CON8		    0x1E
	#define REG_POWER_SUPPLY_CON9		    0x1F
	#define REG_POWER_SUPPLY_CON10		    0x20
	#define REG_POWER_SUPPLY_CON11		    0x24
	#define REG_POWER_SUPPLY_CON12		    0x25
	#define REG_POWER_SUPPLY_CON14		    0x28
	#define REG_OSC_RESIST_CON	  			0x2B
	#define REG_CALIBRATION				    0x2D

    #define REG_ID_CODE1			        0x31
    #define REG_ID_CODE2			        0x32
    #define REG_LINE_INVERS			        0x33
    #define REG_PART_GATE1			        0x34
    #define REG_PART_GATE2			        0x35
    #define REG_GATE_SCAN_SELECT	        0x37
	#define REG_GATE_OUT_CON		        0x3B

    #define REG_GAMMA_CONT12		        0x9A
    #define REG_EXTEND_MODE			        0x9D

    #define REG_OFF_MODE		            0xc0

#endif // _HX8312_H
